@@ -298,7 +298,7 @@ buflist_bar_item_auto_scroll (const char *item_name, int line_number)
 }
 
 /*
- * Returns content of bar item "buffer_plugin": bar item with buffer plugin.
+ * Returns the content of the bar item.
  */
 
 char *
@@ -372,8 +372,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
     snprintf (str_format_number_empty, sizeof (str_format_number_empty),
               "%%-%ds", length_max_number);
 
-    if (buflist_list_buffers[item_index])
-        weechat_arraylist_free (buflist_list_buffers[item_index]);
+    weechat_arraylist_free (buflist_list_buffers[item_index]);
     buflist_list_buffers[item_index] = weechat_arraylist_new (
         16, 0, 1,
         NULL, NULL, NULL, NULL);
@@ -599,8 +598,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
         weechat_hashtable_set (buflist_hashtable_extra_vars,
                                "hotlist",
                                (str_hotlist) ? str_hotlist : "");
-        if (str_hotlist)
-            free (str_hotlist);
+        free (str_hotlist);
 
         /* lag */
         ptr_lag = weechat_buffer_get_string (ptr_buffer, "localvar_lag");
@@ -632,8 +630,7 @@ buflist_bar_item_buflist_cb (const void *pointer, void *data,
             buflist_hashtable_extra_vars,
             buflist_hashtable_options_conditions);
         rc = (condition && (strcmp (condition, "1") == 0));
-        if (condition)
-            free (condition);
+        free (condition);
         if (!rc)
             continue;
 
