@@ -49,17 +49,14 @@ struct t_gui_buffer *
 relay_weechat_protocol_get_buffer (const char *arg)
 {
     struct t_gui_buffer *ptr_buffer;
-    unsigned long value;
     int rc;
 
     ptr_buffer = NULL;
 
     if (strncmp (arg, "0x", 2) == 0)
     {
-        rc = sscanf (arg, "%lx", &value);
-        if ((rc != EOF) && (rc != 0))
-            ptr_buffer = (struct t_gui_buffer *)value;
-        if (ptr_buffer)
+        rc = sscanf (arg, "%p", &ptr_buffer);
+        if ((rc != EOF) && (rc != 0) && ptr_buffer)
         {
             if (!weechat_hdata_check_pointer (
                     relay_hdata_buffer,
@@ -70,6 +67,10 @@ relay_weechat_protocol_get_buffer (const char *arg)
                 ptr_buffer = NULL;
             }
         }
+        else
+        {
+            ptr_buffer = NULL;
+        }
     }
     else
         ptr_buffer = weechat_buffer_search ("==", arg);
@@ -527,7 +528,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(info)
 RELAY_WEECHAT_PROTOCOL_CALLBACK(infolist)
 {
     struct t_relay_weechat_msg *msg;
-    unsigned long value;
+    void *pointer;
     char *args;
     int rc;
 
@@ -536,17 +537,17 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(infolist)
     msg = relay_weechat_msg_new (id);
     if (msg)
     {
-        value = 0;
+        pointer = NULL;
         args = NULL;
         if (argc > 1)
         {
-            rc = sscanf (argv[1], "%lx", &value);
+            rc = sscanf (argv[1], "%p", &pointer);
             if ((rc == EOF) || (rc == 0))
-                value = 0;
+                pointer = NULL;
             if (argc > 2)
                 args = argv_eol[2];
         }
-        relay_weechat_msg_add_infolist (msg, argv[0], (void *)value, args);
+        relay_weechat_msg_add_infolist (msg, argv[0], pointer, args);
         relay_weechat_msg_send (client, msg);
         relay_weechat_msg_free (msg);
     }
@@ -1120,7 +1121,38 @@ relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
                           (unsigned long)ptr_line_data);
                 relay_weechat_msg_add_hdata (
                     msg, cmd_hdata,
-                    "buffer,date,date_usec,date_printed,date_usec_printed,"
+                    "buffer,id,date,date_usec,date_printed,date_usec_printed,"
+                    "displayed,notify_level,highlight,tags_array,"
+                    "prefix,message");
+                relay_weechat_msg_send (ptr_client, msg);
+                relay_weechat_msg_free (msg);
+            }
+        }
+    }
+    else if (strcmp (signal, "buffer_line_data_changed") == 0)
+    {
+        ptr_line_data = (struct t_gui_line_data *)signal_data;
+        if (!ptr_line_data)
+            return WEECHAT_RC_OK;
+
+        ptr_buffer = weechat_hdata_pointer (relay_hdata_line_data, ptr_line_data,
+                                            "buffer");
+        if (!ptr_buffer || relay_buffer_is_relay (ptr_buffer))
+            return WEECHAT_RC_OK;
+
+        /* send signal only if sync with flag "buffer" */
+        if (relay_weechat_protocol_is_sync (ptr_client, ptr_buffer,
+                                            RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER))
+        {
+            msg = relay_weechat_msg_new (str_signal);
+            if (msg)
+            {
+                snprintf (cmd_hdata, sizeof (cmd_hdata),
+                          "line_data:0x%lx",
+                          (unsigned long)ptr_line_data);
+                relay_weechat_msg_add_hdata (
+                    msg, cmd_hdata,
+                    "buffer,id,date,date_usec,date_printed,date_usec_printed,"
                     "displayed,notify_level,highlight,tags_array,"
                     "prefix,message");
                 relay_weechat_msg_send (ptr_client, msg);
